use crate::world::level_map::Indexer;

#[derive(Copy, Clone, Debug, Default)]
pub enum GenerationDirection {
	Top,
	Bottom,
	Left,
	Right,
	#[default]
	None,
}

pub fn draw_box(
	start_x: usize,
	start_y: usize,
	width: usize,
	height: usize,
	value: usize,
	layer: &mut Vec<usize>,
	indexer: &Indexer,
) {
	if !validate_box(start_x, start_y, width, height, layer, indexer) {
		return;
	}

	for y in start_y..(start_y + height) {
		for x in start_x..(start_x + width) {
			let idx = indexer.index(x, y);
			layer[idx] = value;
		}
	}
}

pub fn validate_box(
	start_x: usize,
	start_y: usize,
	width: usize,
	height: usize,
	layer: &Vec<usize>,
	indexer: &Indexer,
) -> bool {
	let map_width = indexer.width();
	let map_height = indexer.height();

	if start_y + height >= map_height {
		return false;
	}

	if start_x + width >= map_width {
		return false;
	}

	for x in start_x..(start_x + width) {
		for y in start_y..(start_y + height) {
			let idx = indexer.index(x, y);
			if layer[idx] != 0 {
				return false;
			}
		}
	}

	true
}
